"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var gaze = require("gaze");
var glob_1 = require("glob");
var sharp = require("sharp");
var lodash_1 = require("lodash");
var shortid_1 = require("shortid");
var config_1 = require("../config/config");
var source = path.resolve(config_1.default.slides);
console.log("Loading slideshow from directory: " + source);
var filepattern = path.join(source, '**/*.jpg');
var fileToId = {};
var idToFile = {};
glob_1.sync(filepattern, { nocase: true }).forEach(function (filename) {
    var id = shortid_1.generate();
    fileToId[filename] = id;
    idToFile[id] = filename;
});
function randomIndex(length) {
    return Math.floor(Math.random() * length);
}
// TODO (#2): create a playlist per connection/session, to guarantee
// good independent shuffling for multiple clients.
var playlist = lodash_1.shuffle(fileToId);
var index = 0;
gaze(path.relative(process.cwd(), filepattern), function (err, watcher) {
    watcher.on('added', function (filepath) {
        console.log("File added: " + filepath);
        var id = shortid_1.generate();
        fileToId[filepath] = id;
        idToFile[id] = filepath;
        var location = randomIndex(playlist.length + 1);
        playlist.splice(location, 0, id);
        if (location < index) {
            index = index + 1;
        }
    });
    watcher.on('deleted', function (filepath) {
        console.log("File deleted: " + filepath);
        var id = fileToId[filepath];
        delete fileToId[filepath];
        delete idToFile[id];
        var location = playlist.indexOf(id);
        playlist.splice(location, 1);
        if (location < index) {
            index = index - 1;
        }
    });
});
function nextImage() {
    index = index + 1;
    if (index >= playlist.length) {
        console.log("End of playlist reached. Reshuffling...");
        var previous = playlist[index - 1];
        playlist = lodash_1.shuffle(fileToId);
        index = 0;
        // When looping, ensure that the first item in the next playlist is not
        // the same as the last item in the previous playlist (otherwise the same
        // item would appear twice in a row).
        if (playlist[0] === previous) {
            var replacement = randomIndex(playlist.length - 1) + 1;
            playlist[0] = playlist[replacement];
            playlist[replacement] = previous;
        }
    }
    var id = playlist[index];
    var file = idToFile[id];
    console.log("Next image: " + id + " (" + file + ")");
    return id;
}
var ImagesController = /** @class */ (function () {
    function ImagesController() {
    }
    ImagesController.prototype.next = function (req, res) {
        res.json({ value: nextImage() });
    };
    ImagesController.prototype.image = function (req, res) {
        return __awaiter(this, void 0, void 0, function () {
            var id, file, buffer, error_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        id = req.params.id;
                        file = idToFile[id];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, sharp(file)
                                .rotate()
                                .jpeg()
                                .toBuffer()];
                    case 2:
                        buffer = _a.sent();
                        res.contentType('image/jpeg');
                        res.send(buffer);
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        console.error("An error occurred when rotating the file '" + id + "': " + error_1);
                        // If we can't rotate for some reason just send the file as-is.
                        res.sendFile(file);
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    ImagesController.prototype.indexStatic = function (req, res) {
        return __awaiter(this, void 0, void 0, function () {
            var width, height, id, file, img, composite, error_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        width = parseInt(req.query.width, 10);
                        height = parseInt(req.query.height, 10);
                        id = nextImage();
                        file = idToFile[id];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, sharp(file)
                                .rotate()
                                .resize({
                                width: width,
                                height: height,
                                fit: 'inside',
                                withoutEnlargement: true
                            })
                                .raw()
                                .toBuffer({
                                resolveWithObject: true
                            })];
                    case 2:
                        img = _a.sent();
                        return [4 /*yield*/, sharp({
                                create: {
                                    width: width,
                                    height: height,
                                    channels: 3,
                                    background: '#111'
                                }
                            })
                                .composite([
                                {
                                    input: img.data,
                                    raw: img.info
                                }
                            ])
                                .jpeg()
                                .toBuffer()];
                    case 3:
                        composite = _a.sent();
                        res.contentType('image/jpeg');
                        res.send(composite);
                        return [3 /*break*/, 5];
                    case 4:
                        error_2 = _a.sent();
                        console.error("An error occurred when resizing the file '" + id + "': " + error_2);
                        // If we can't rotate/resize for some reason just send the file as-is.
                        res.sendFile(file);
                        return [3 /*break*/, 5];
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    return ImagesController;
}());
exports.default = ImagesController;
exports.imagesController = new ImagesController();
//# sourceMappingURL=images.server.controller.js.map